<?php

namespace FM\Service;

use FM\Factory\Fm00020000SolicitacoesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Fm00020000SolicitacoesService
{

    protected $SolicitacaoFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(Fm00020000SolicitacoesFactory $SolicitacaoFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->SolicitacaoFactory           = $SolicitacaoFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    // Busca as solicitações
    public function getRequests($params)
    {
        return $this->SolicitacaoFactory->getRequests();
    }

    // Busca os tipos de solicitação
    public function getRequestTypes()
    {
        return $this->SolicitacaoFactory->getRequestTypes();
    }

    // Busca as solicitações para o grid
    public function getTipoSolicitacao()
    {
        return $this->SolicitacaoFactory->getTipoSolicitacao();
    }

    // Busca as informações de uma solicitação específica
    public function getRequestInfo($nrsolicitacao)
    {
        $requestInfo = $this->SolicitacaoFactory->getRequestInfo($nrsolicitacao);
        return $requestInfo;
    }

    // Busca o histórico de uma solicitação
    public function getRequestHistoric($parametros)
    {
        return $this->SolicitacaoFactory->getRequestHistoric(array(
            "NRSOLICITACAO" => $parametros['NRSOLICITACAO'],
        ));
    }

    // Insere e edita os Tipos de Solicitações
    public function insertRequestType($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            foreach ($params["PARAMS"] as $row) {
                if ($row["NRTIPOSOLICITACAO"] != 0) {
                    if (strlen($row['IDPRIORIDADEPADR']) > 1) {
                        if ($row['IDPRIORIDADEPADR'] == "Baixa") {
                            $row['IDPRIORIDADEPADR'] = "1";
                        } elseif ($row['IDPRIORIDADEPADR'] == "Média") {
                            $row['IDPRIORIDADEPADR'] = "2";
                        } elseif ($row['IDPRIORIDADEPADR'] == "Alta") {
                            $row['IDPRIORIDADEPADR'] = "3";
                        } else {
                            $row['IDPRIORIDADEPADR'] = "4";
                        }
                    }
                    $typeParams = array(
                        'NRTIPOSOLICITACAO' => $row['NRTIPOSOLICITACAO'],
                        'NMTIPOSOLICITACAO' => $row['NMTIPOSOLICITACAO'],
                        'IDPRIORIDADEPADR'  => $row['IDPRIORIDADEPADR'],
                        'CDOPERULTATU'      => $operador,
                        'NRORGULTATU'       => $nrorg,
                        'IDATIVO'           => $row['IDATIVO']
                    );

                    $this->SolicitacaoFactory->updateRequestType($typeParams);
                } else {
                    $nrtiposolicitacao  = $this->newCode->getCodeByTableName('FCM_TIPOSOLICITACAO', $this->Environment->getNrOrg(), 1, 10, true);
                    $typeParams = array(
                        'NRTIPOSOLICITACAO' => $nrtiposolicitacao,
                        'NMTIPOSOLICITACAO' => $row['NMTIPOSOLICITACAO'],
                        'IDPRIORIDADEPADR'  => $row['IDPRIORIDADEPADR'],
                        'NRORG'             => $nrorg,
                        'CDOPERINCLUSAO'    => $operador,
                        'NRORGINCLUSAO'     => $nrorg,
                    );

                    $this->SolicitacaoFactory->insertRequestType($typeParams);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarTipoSolicitacao($params)
    {
        $NR = implode('_', $params["NRS"]);
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            $typeParams = array(
                'NRTIPOSOLICITACAO' => $NR,
                'IDATIVO'           => 'N',
                'CDOPERULTATU'      => $operador,
                'NRORGULTATU'       => $nrorg
            );

            $this->SolicitacaoFactory->inativarTipoSolicitacao($typeParams);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Processo de clasificação -> Alteração nas tabelas de solicitação/sol_tarefas, checklistAgend e  histórico
    public function classifyRequest($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->SolicitacaoFactory->updateRequest(array(
                'NRSOLICITACAO'     => $params['NRSOLICITACAO'],
                'NMSOLICITACAO'     => $params['NMSOLICITACAO'],
                'IDSITUACAO'        => '2',
                'NRTIPOSOLICITACAO' => $params['NRTIPOSOLICITACAO'],
                'IDPRIORIDADE'      => $params['IDPRIORIDADE'],
                'CDOPERRESPONSAVEL' => $params['CDOPERRESPONSAVEL'],
                'DSSOLICITACAO'     => $params['DSSOLICITACAO'],
                'CDOPERULTATU'      => $cdOperador
            ));

            
            $this->SolicitacaoFactory->insertHistoricData(array(
                'NRSOLICITACAOHISTORICO'    => $this->newCode->getCodeByTableName('FCM_SOLICITACAOHISTORICO', $nrorg, 1, 10, true),
                'NRSOLICITACAO'             => $params['NRSOLICITACAO'],
                'NMSOLICITACAO'             => $params['NMSOLICITACAO'],
                'IDSITUACAO'                => '2',
                'IDTIPOHISTORICO'           => '2',
                'CDOPERSOLICITANTE'         => $params['CDOPERRESPONSAVEL'],
                'DSSOLICITACAO'             => 'A solicitação foi classificada e uma nova tarefa foi criada.',
                'NRORG'                     => $nrorg,
                'CDOPERULTATU'              => $cdOperador
            ));
            
            $nrChecklistAgend  = $this->newCode->getCodeByTableName('FCM_CHECKLISTAGEND', $nrorg, 1, 10, true);

            $this->insertNovaTarefa(array(
                "NRCHECKLISTAGEND"  => $nrChecklistAgend,
                "NMTAREFA"          => $params['NMSOLICITACAO'],
                "LOCAL"             => [$params['NRLOCALCLIENTE']],
                "CHECKLIST"         => $params['NRCHECKLIST'],
                "CDOPERADOR"        => $params['CDOPERRESPONSAVEL'],
                "NRLOCALCLIENTE"    => $params['NRLOCALCLIENTE'],
                "NRCHECKLIST"       => $params['NRCHECKLIST'],
                "DATAINICIAL"       => (date('I') == 1) ? date("Y/m/d H:i", strtotime('-1 Hours')) : date('Y/m/d H:i'),
                "DATAFINAL"         => (date('I') == 1) ? date("Y/m/d H:i", strtotime('-1 Hours +30 Minutes')) : date('Y/m/d H:i', strtotime('+30 Minutes')),
            ));
        
            $this->SolicitacaoFactory->createTaskRequest(array(
                'NRSOLICITACAOTAREFAS'  => $this->newCode->getCodeByTableName('FCM_SOLICITACAOTAREFAS', $nrorg, 1, 10, true),
                'NRSOLICITACAO'         => $params['NRSOLICITACAO'],
                'NRCHECKLISTAGEND'      => $nrChecklistAgend,
                'NRORG'                 => $nrorg,
                'CDOPERULTATU'          => $cdOperador
            )); // Cria uma relação de tarefa com solicitação

            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertNovaTarefa($param)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG =  $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->SolicitacaoFactory->insertNovaTarefa(array(
                "NRCHECKLISTAGEND"  => $param['NRCHECKLISTAGEND'],
                "NMCHECKLISTAGEND"  => $param['NMTAREFA'],
                "NRLOCALCLIENTE"    => $param['NRLOCALCLIENTE'],
                "CDOPERADOR"        => $param['CDOPERADOR'],
                "IDSITUACAOAGEND"   => 1,
                "DTHORAAGEND"       => $param['DATAINICIAL'],
                "DTHRFINALAGEND"    => $param['DATAFINAL'],
                "NRCHECKLIST"       => $param['NRCHECKLIST'],
                "CDOPERINCLUSAO"    => $cdOperador,
                "CDOPERULTATU"      => $cdOperador,
                "NRORG"             => $NRORG
            ));

            $tarefas = array();
            if ($param['NRCHECKLIST'] !== null) {
                $tarefas = $this->SolicitacaoFactory->getTarefaChk(array("NRCHECKLIST" =>  $param['NRCHECKLIST']));
            }

            $this->SolicitacaoFactory->insertNovaTarefa2(array(
                "NRTAREFAAGEND"     => $this->newCode->getCodeByTableName("FCM_TAREFAAGEND", $NRORG, 1, 10, true),
                "NRCHECKLISTAGEND"  => $param['NRCHECKLISTAGEND'],
                "NRTAREFACHK"       => (count($tarefas) > 0) ? $tarefas[0]["NRTAREFACHK"] : null,
                "CDOPERADOR"        => $param['CDOPERADOR'],
                "DTHORAAGEND"       => $param['DATAINICIAL'],
                "DSOBSTAREFAAGEND"  => null,
                "CDOPERINCLUSAO"    => $cdOperador,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getLocalRequest($params)
    {
        return $this->SolicitacaoFactory->getLocalRequest($params);
    }

    public function newTarefa($params)
    {
        return $this->SolicitacaoFactory->newTarefa($params);
    }

    public function newTarefaChk($params)
    {
        return $this->SolicitacaoFactory->newTarefaChk($params);
    }

    public function setNewRequest($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG =  $this->Environment->getNrOrg();
            $NRSOLICITACAO = $this->newCode->getCodeByTableName("FCM_SOLICITACAO", $NRORG, 1, 10, true);

            $params = array(
                "NRSOLICITACAO"  => $NRSOLICITACAO,
                "NMSOLICITACAO"  => $parametros['NMSOLICITACAO'],
                "IDSITUACAO"     => $parametros['IDSITUACAO'],
                "NRLOCALCLIENTE" => $parametros['NRLOCALCLIENTE'],
                "NRFUNCABERTURA" => $parametros['NRFUNCABERTURA'],
                "DSSOLICITACAO"  => $parametros['DSSOLICITACAO'],
                "NRORG"          => $NRORG
            );

            $this->SolicitacaoFactory->insertNewRequest($params);

            $NRSOLICITACAOHISTORICO = $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true);

            $params = array(
                "NRSOLICITACAOHISTORICO" => $NRSOLICITACAOHISTORICO,
                "NRSOLICITACAO"          => $NRSOLICITACAO,
                "IDSITUACAO"             => 1,
                "IDTIPOHISTORICO"        => 1,
                "NRFUNCSOLICITANTE"      => $parametros['NRFUNCABERTURA'],
                "DSSOLICITACAO"          => $parametros['DSSOLICITACAO'],
                "IDSATISFACAOSOLICITA"   => null,
                "NRORG"                  => $NRORG
            );
            $this->SolicitacaoFactory->insertRequestHistory($params);

            $this->connection->commit();
            $response = "OK";
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response = ($e->getMessage());
        }
        return $response;
    }

    public function getCardsRequests($parametro, $parametroIds)
    {

        $param = '';

        if ($parametro['selectName'] != '' && $parametro['selectName'] != null) {
            $param = $param . " AND S.NRSOLICITACAO = " . $parametro['selectName'];
        }

        if ($parametro['selectLocal'] != '' && $parametro['selectLocal'] != null) {
            $param = $param . " AND S.NRLOCALCLIENTE = " . $parametro['selectLocal'];
        }

        if ($parametro['selectSituation'] != '' && $parametro['selectSituation'] != null) {
            $param = $param . " AND S.IDSITUACAO = " . $parametro['selectSituation'];
        }

        if ($parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'] != '' && $parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'] != null && strlen($parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')']) == 21) {
            $param = $param . " AND TO_CHAR(S.DTSOLICITACAO,'DD/MM/YYYY') = " . $parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'];
        }

        $usersIds = implode(',', $parametroIds); // Transforma o array em uma string com os ids

        $cards = $this->SolicitacaoFactory->getCardsRequests($param, $usersIds);
        return $cards;
    }

    public function showRequest($params)
    {
        $parametros = array(
            "NRSOLICITACAO"    => $params['NRSOLICITACAO'],
        );

        $details = $this->SolicitacaoFactory->showRequest($parametros);
        return $details;
    }

    public function getSituationRequest()
    {

        $situation = $this->SolicitacaoFactory->getSituationRequest();
        return $situation;
    }

    public function getNameRequest()
    {

        $nameRequest = $this->SolicitacaoFactory->getNameRequest();
        return $nameRequest;
    }

    public function evaluateRequest($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG =  $this->Environment->getNrOrg();
            $params = array(
                "NRSOLICITACAO"        => $parametros['NRSOLICITACAO'],
                "IDSITUACAO"           => ($parametros['CONDITION']) ? 4 : 1,
                "DSSOLICITACAO"        => $parametros['DSSOLICITACAO']
            );
            $this->SolicitacaoFactory->upRequest($params);

            $NRSOLICITACAOHISTORICO = $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true);

            $params = array(
                "NRSOLICITACAOHISTORICO" => $NRSOLICITACAOHISTORICO,
                "NRSOLICITACAO"          => $parametros['NRSOLICITACAO'],
                "IDSITUACAO"             => ($parametros['CONDITION']) ? 4 : 1,
                "IDTIPOHISTORICO"        => 4,
                "NRFUNCSOLICITANTE"      => $parametros['NRFUNCABERTURA'],
                "DSSOLICITACAO"          => $parametros['DSSOLICITACAO'],
                "IDSATISFACAOSOLICITA"   => $parametros['IDSATISFACAOSOLICITA'],
                "NRORG"                  => $NRORG
            );
            $this->SolicitacaoFactory->insertRequestHistory($params);

            $this->connection->commit();
            $response = "OK";
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response = ($e->getMessage());
        }
        return $response;
    }

    public function encerrarSolicitacao($parametros)
    {
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();

            $parametrosAtualiza = array(
                "NRSOLICITACAO" => $parametros["NRSOLICITACAO"],
                "IDSITUACAO"    => 4
            );

            $this->SolicitacaoFactory->atualizaSituacao($parametrosAtualiza);

            $NRSOLICITACAOHISTORICO = $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $nrorg, 1, 10, true);
            $parametrosHistorico = array (
                "NRSOLICITACAOHISTORICO" => $NRSOLICITACAOHISTORICO,
                "NRSOLICITACAO"          => $parametros["NRSOLICITACAO"],
                "NMSOLICITACAO"          => $parametros["NMSOLICITACAO"],
                "IDSITUACAO"             => 4,
                "IDTIPOHISTORICO"        => 4,
                "DSSOLICITACAO"          => "Solicitação encerrada.",
                "NRORG"                  => $nrorg,
                "CDOPERULTATU"           => $cdOperador,
                "CDOPERSOLICITANTE"      => $parametros["CDOPERSOLICITANTE"]
            );

            $this->SolicitacaoFactory->insereHistorico($parametrosHistorico);
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
