<?php

namespace FM\Service;

use FM\Factory\FmtOrdemServFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FileServerService;

class FmtOrdemServService
{
    protected $connection;
    protected $OrdemServFactory;
    protected $Environment;
    protected $newCode;
    protected $FileServerService;
    protected $fileServerUrl;

    public function __construct(EntityManager $entityManager, FmtOrdemServFactory $OrdemServFactory, Environment $Environment, NewCode $newCode, FileServerService $FileServerService, $fileServerUrl)
    {
        $this->OrdemServFactory  = $OrdemServFactory;
        $this->connection        = $entityManager->getConnection();
        $this->Environment       = $Environment;
        $this->newCode           = $newCode;
        $this->FileServerService = $FileServerService;
        $this->fileServerUrl     = $fileServerUrl;
    }

    public function getOrdensServico()
    {
        return $this->OrdemServFactory->getOrdensServico();
    }

    public function getComentariosOrdem($parametros)
    {
        $parametroComentario = array(
            "NRORDEMSERV" => $parametros["NRORDEMSERV"]
        );

        return $this->OrdemServFactory->getComentariosOrdem($parametroComentario);
    }

    public function saveComentarioOrdem($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $NRCOMENTORDEMSERV = $this->newCode->getCodeByTableName("FCM_COMENTORDEMSERV", $nrorg, 1, 10, true);

            $parametroComentario = array(
                "NRCOMENTORDEMSERV" => $NRCOMENTORDEMSERV,
                "NRORDEMSERV"       => $parametros["NRORDEMSERV"],
                "DSCOMENTARIO"      => $parametros["DSCOMENTARIO"]
            );

            $this->OrdemServFactory->saveComentarioOrdem($parametroComentario);

            $this->connection->commit();
        } catch (\Exception $e) {
            var_dump($e->getMessage());
            $this->connection->rollback();
        }
    }

    public function insertOrdemServico($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $NRORDEMSERV = $this->newCode->getCodeByTableName("FCM_ORDEMSERV", 20, 1, 10, true);

            $parametroOrdem = array(
                "NRORDEMSERV"       => $NRORDEMSERV,
                "NMORDEMOSERV"      => $parametros["NMORDEMOSERV"],
                "NREQUIPAMENTO"     => $parametros["NREQUIPAMENTO"],
                "NRCATEGORIAMANUT"  => $parametros["NRCATEGORIAMANUT"],
                "NRTIPOMANUTENCAO"  => $parametros["NRTIPOMANUTENCAO"],
                "NRPRIORIDADEMANUT" => $parametros["NRPRIORIDADEMANUT"],
                "DTORDEMSERV"       => $parametros["DTORDEMSERV"],
                "IDSITUACAO"        => $parametros["IDSITUACAO"],
                "DSORDEMSERV"       => $parametros["DSORDEMSERV"],
                "IDFUNCEXTERNO"     => $parametros["IDFUNCEXTERNO"],
                "NRORG"             => $nrorg
            );

            $this->OrdemServFactory->insertOrdemServico($parametroOrdem);

            if ($parametros["SOLICITACAO"]) {
                $NRRELSOLICITAORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELSOLICITAORDEMSERV", $nrorg, 1, 10, true);

                $parametroRelacionamento = array(
                    "NRRELSOLICITAORDEMSERV"  => $NRRELSOLICITAORDEMSERV,
                    "NRORDEMSERV"             => $NRORDEMSERV,
                    "NRSOLICITACAOSERV"       => $parametros["NRSOLICITACAOSERV"]
                );

                $this->OrdemServFactory->insertRelSolicitacaoOrdem($parametroRelacionamento);
            }

            foreach ($parametros["NRESCOPOS"] as $NRESCOPOMANUT) {
                $NRRELESCORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELESCORDEMSERV", $nrorg, 1, 10, true);

                $parametroEscopo = array(
                    "NRRELESCORDEMSERV" => $NRRELESCORDEMSERV,
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "NRESCOPOMANUT"     => $NRESCOPOMANUT
                );

                $this->OrdemServFactory->insertRelEscopoOrdem($parametroEscopo);
            }

            if (isset($parametros["FOTOS"])) {
                $parametroFotos = array(
                    "NRORDEMSERV"   => $NRORDEMSERV,
                    "FOTOS"         => $parametros["FOTOS"]
                );
                $this->insertFotosOrdem($parametroFotos);
            }

            if (isset($parametros["DSCOMENTARIO"])) {
                $parametroComentario = array(
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "DSCOMENTARIO"      => $parametros["DSCOMENTARIO"]
                );

                $this->saveComentarioOrdem($parametroComentario);
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            var_dump($ex);
            $this->connection->rollback();
        }
    }

    public function insertFotosOrdem($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach ($parametros["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_manutencao_fm");

                $NRIMGORDEMSERV = $this->newCode->getCodeByTableName('FCM_IMGORDEMSERV', $nrorg, 1, 10, true);
                $parametroFoto = array(
                    "NRIMGORDEMSERV"    => $NRIMGORDEMSERV,
                    "NRORDEMSERV"       => $parametros["NRORDEMSERV"],
                    "DSCAMINHO"         => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG"             => $nrorg,
                    "DSTYPE"            => $img["type"],
                    "NRSIZE"            => $img["size"]
                );

                $this->OrdemServFactory->insertFotoOrdem($parametroFoto);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }
}
