<?php

namespace FM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use FM\Factory\FmtParametrizacaoFactory;

class FmtParametrizacaoService
{
    protected $Environment;
    protected $newCode;
    protected $ParametrizacaoFactory;
    protected $connection;

    public function __construct(EntityManager $EntityManager, Environment $Environment, NewCode $newCode, FmtParametrizacaoFactory $ParametrizacaoFactory)
    {
        $this->connection = $EntityManager->getConnection();
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->ParametrizacaoFactory = $ParametrizacaoFactory;
    }

    /*
     * TAB CATEGORIA DE MANUTENÇÃO
     */

    public function saveCategoriaManutencao($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["ROWS"] as $row) {
                if ($row["NRCATEGORIAMANUT"] == 0) {
                    $NRCATEGORIAMANUT = $this->newCode->getCodeByTableName("FCM_CATEGORIAMANUT", $nrorg, 1, 10, true);

                    $parametrosCategoria = array(
                        "NRCATEGORIAMANUT"  => $NRCATEGORIAMANUT,
                        "NMCATEGORIAMANUT"  => $row['NMCATEGORIAMANUT'],
                        "NMABREVCATEGMANUT" => $row['NMABREVCATEGMANUT'],
                        "NRORG"             => $nrorg,
                        "CDOPERINCLUSAO"    => $operador,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->ParametrizacaoFactory->insertCategoriaManutencao($parametrosCategoria);
                } else {
                    $parametrosCategoria = array(
                        "NRCATEGORIAMANUT"  => $row['NRCATEGORIAMANUT'],
                        "NMCATEGORIAMANUT"  => $row['NMCATEGORIAMANUT'],
                        "NMABREVCATEGMANUT" => $row['NMABREVCATEGMANUT'],
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->ParametrizacaoFactory->editCategoriaManutencao($parametrosCategoria);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function getCategoriasManutencao()
    {
        $data = $this->ParametrizacaoFactory->getCategoriasManutencao();

        foreach ($data['data'] as &$row) {
            $row['NRFUNCIONARIO'] = explode(', ', $row['NRFUNCIONARIO']);
        }

        return $data;
    }

    public function getTecnicosRegistrados()
    {
        return $this->ParametrizacaoFactory->getTecnicosRegistrados();
    }

    public function deleteCategoriasManutencao($params)
    {
        $parametro = implode(',', $params["NRS"]);
        $this->ParametrizacaoFactory->deleteCategoriasManutencao($parametro);
    }

    public function getAbrevCategoria()
    {
        return $this->ParametrizacaoFactory->getAbrevCategoria();
    }

    /*
     * TAB TIPO DE MANUTENÇÃO
     */

    public function saveTipoManutencao($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["ROWS"] as $row) {
                if ($row["NRTIPOMANUTENCAO"] == 0) {
                    $NRTIPOMANUTENCAO = $this->newCode->getCodeByTableName("FCM_TIPOMANUTENCAO", $nrorg, 1, 10, true);

                    $parametrosTipo = array(
                        "NRTIPOMANUTENCAO"  => $NRTIPOMANUTENCAO,
                        "NMTIPOMANUTENCAO"  => $row['NMTIPOMANUTENCAO'],
                        "NRORG"             => $nrorg,
                        "CDOPERINCLUSAO"    => $operador,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->ParametrizacaoFactory->insertTipoManutencao($parametrosTipo);
                } else {
                    $parametrosTipo = array(
                        "NRTIPOMANUTENCAO"  => $row['NRTIPOMANUTENCAO'],
                        "NMTIPOMANUTENCAO"  => $row['NMTIPOMANUTENCAO'],
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->ParametrizacaoFactory->editTipoManutencao($parametrosTipo);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function getTiposManutencao()
    {
        return $this->ParametrizacaoFactory->getTiposManutencao();
    }

    public function deleteTiposManutencao($params)
    {
        $parametro = implode(',', $params["NRS"]);
        $this->ParametrizacaoFactory->deleteTiposManutencao($parametro);
    }

    /*
     * TAB PRIORIDADES
     */
    
    public function savePrioridade($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["ROWS"] as $row) {
                if ($row["NRPRIORIDADEMANUT"] == 0) {
                    $NRPRIORIDADEMANUT = $this->newCode->getCodeByTableName("FCM_PRIORIDADEMANUT", $nrorg, 1, 10, true);
                    $corPrioridade = isset($row['DSCORPRIORIDADE']) ? $row['DSCORPRIORIDADE'] : "#000000";

                    $parametrosPrioridade = array(
                        "NRPRIORIDADEMANUT" => $NRPRIORIDADEMANUT,
                        "NMPRIORIDADEMANUT" => $row['NMPRIORIDADEMANUT'],
                        "DSCORPRIORIDADE"   => $corPrioridade,
                        "NRORG"             => $nrorg,
                        "CDOPERINCLUSAO"    => $operador,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );
                    
                    $this->ParametrizacaoFactory->insertPrioridade($parametrosPrioridade);
                    
                    if (isset($row['NRIMPACTOMANUT'])) {
                        foreach ($row['NRIMPACTOMANUT'] as $impacto) {
                            $this->insertImpactoRelacionado($NRPRIORIDADEMANUT, $impacto, $nrorg, $operador);
                        }
                    }
                } else {
                    $parametrosPrioridade = array(
                        "NRPRIORIDADEMANUT" => $row["NRPRIORIDADEMANUT"],
                        "NMPRIORIDADEMANUT" => $row["NMPRIORIDADEMANUT"],
                        "DSCORPRIORIDADE"   => $row["DSCORPRIORIDADE"],
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->ParametrizacaoFactory->editPrioridade($parametrosPrioridade);

                    $impactosRelacionados = $row['NRIMPACTOMANUT'];

                    foreach ($impactosRelacionados as $impacto) {
                        $parametrosRelacionamento = array(
                            "NRPRIORIDADEMANUT" => $row["NRPRIORIDADEMANUT"],
                            "NRIMPACTOMANUT"    => $impacto
                        );

                        $numRelacionamentos = $this->ParametrizacaoFactory->verificaRelPrioridadeImpacto($parametrosRelacionamento);

                        if ($numRelacionamentos == 0) {
                            $this->insertImpactoRelacionado($row["NRPRIORIDADEMANUT"], $impacto, $nrorg, $operador);
                        }
                    }

                    $parametroImpactosDesatualizados = array(
                        "NRPRIORIDADEMANUT" => $row["NRPRIORIDADEMANUT"]
                    );
                    $impactosDesatualizados = $impactosRelacionados ? implode(',', $impactosRelacionados) : "-1";
                    $this->ParametrizacaoFactory->deleteImpactosDesatualizados($parametroImpactosDesatualizados, $impactosDesatualizados);
                    $this->ParametrizacaoFactory->updatePrioridadePadrao($parametroImpactosDesatualizados, $impactosDesatualizados);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function insertImpactoRelacionado($NRPRIORIDADEMANUT, $NRIMPACTOMANUT, $nrorg, $operador)
    {
        $RELPRIORIDADEIMPACTO = $this->newCode->getCodeByTableName("FCM_RELPRIORIDADEIMPACTO", $nrorg, 1, 10, true);

        $parametrosImpactoRelacionado = array(
            "RELPRIORIDADEIMPACTO"  => $RELPRIORIDADEIMPACTO,
            "NRPRIORIDADEMANUT"     => $NRPRIORIDADEMANUT,
            "NRIMPACTOMANUT"        => $NRIMPACTOMANUT,
            "NRORG"                 => $nrorg,
            "CDOPERINCLUSAO"        => $operador,
            "NRORGINCLUSAO"         => $nrorg,
            "CDOPERULTATU"          => $operador,
            "NRORGULTATU"           => $nrorg
        );

        $this->ParametrizacaoFactory->insertImpactoRelacionado($parametrosImpactoRelacionado);
    }

    public function getPrioridadesManutencao()
    {
        $data = $this->ParametrizacaoFactory->getPrioridadesManutencao();

        foreach ($data["data"] as &$row) {
            if (isset($row["NRIMPACTOMANUT"]) && $row["NRIMPACTOMANUT"][0] != "") {
                foreach ($row["NRIMPACTOMANUT"] as &$NR) {
                    $NR = (int) $NR;
                }
            }
        }

        return $data;
    }

    public function getImpactosRelacionados()
    {
        return $this->ParametrizacaoFactory->getImpactosRelacionados();
    }

    public function deletePrioridades($params)
    {
        $parametro = implode(',', $params["NRS"]);
        $numRelOrdemServ = $this->ParametrizacaoFactory->verificaRelOrdemServ($parametro);
        $numRelSolictServ = $this->ParametrizacaoFactory->verificaRelSolicitServ($parametro);
        
        if ($numRelOrdemServ == 0 && $numRelSolictServ == 0) {
            $this->ParametrizacaoFactory->deleteImpactosRelacionados($parametro);
        }
        $this->ParametrizacaoFactory->deletePrioridades($parametro);
    }

    /*
     * TAB IMPACTO
     */

    public function saveImpacto($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["ROWS"] as $row) {
                if ($row["NRIMPACTOMANUT"] == 0) {
                    $NRIMPACTOMANUT = $this->newCode->getCodeByTableName("FCM_IMPACTOMANUT", $nrorg, 1, 10, true);
                    $prioridadePadrao = isset($row['NMPRIORIDADEMANUT']) ? $row['NMPRIORIDADEMANUT'] : "";

                    $parametrosImpacto = array(
                        "NRIMPACTOMANUT"        => $NRIMPACTOMANUT,
                        "NMIMPACTOMANUT"        => $row['NMIMPACTOMANUT'],
                        "NRPRIORIDADEMANUTPADR" => $prioridadePadrao,
                        "NRORG"                 => $nrorg,
                        "CDOPERINCLUSAO"        => $operador,
                        "NRORGINCLUSAO"         => $nrorg,
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg
                    );

                    $this->ParametrizacaoFactory->insertImpacto($parametrosImpacto);
                } else {
                    $parametrosImpacto = array(
                        "NRIMPACTOMANUT"        => $row['NRIMPACTOMANUT'],
                        "NMIMPACTOMANUT"        => $row['NMIMPACTOMANUT'],
                        "NRPRIORIDADEMANUTPADR" => $row['NRPRIORIDADEMANUTPADR'],
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg
                    );

                    $this->ParametrizacaoFactory->editImpacto($parametrosImpacto);
                }

                if (isset($row['NRPRIORIDADEMANUTPADR'])) {
                    $this->criaRelacinamento($row);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function criaRelacinamento($row)
    {
        $nrorg = $this->Environment->getNrorg();
        $operador = $this->Environment->getCdOperador();

        $parametrosRel = array(
            "NRIMPACTOMANUT"    => $row['NRIMPACTOMANUT'],
            "NRPRIORIDADEMANUT" => $row['NRPRIORIDADEMANUTPADR']
        );

        $retorno = $this->ParametrizacaoFactory->verificaRelImpactoPrioridade($parametrosRel);

        if ($retorno == 0) {
            $RELPRIORIDADEIMPACTO = $this->newCode->getCodeByTableName("FCM_RELPRIORIDADEIMPACTO", $nrorg, 1, 10, true);

            $parametrosImpactoRelacionado = array(
                "RELPRIORIDADEIMPACTO"  => $RELPRIORIDADEIMPACTO,
                "NRPRIORIDADEMANUT"     => $row['NRPRIORIDADEMANUTPADR'],
                "NRIMPACTOMANUT"        => $row['NRIMPACTOMANUT'],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"        => $operador,
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERULTATU"          => $operador,
                "NRORGULTATU"           => $nrorg
            );

            $this->ParametrizacaoFactory->insertImpactoRelacionado($parametrosImpactoRelacionado);
        }
    }

    public function getImpactosManutencao()
    {
        $data = $this->ParametrizacaoFactory->getImpactosManutencao();

        return $data;
    }

    public function getPrioridades()
    {
        return $this->ParametrizacaoFactory->getPrioridades();
    }

    public function deleteImpactos($params)
    {
        $parametro = implode(',', $params["NRS"]);
        $this->ParametrizacaoFactory->deleteImpactos($parametro);
    }

    public function getImpactos()
    {
        return $this->ParametrizacaoFactory->getImpactos();
    }

    /*
     * TAB ESCOPO
     */

    public function saveEscopo($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["ROWS"] as $row) {
                if ($row["NRESCOPOMANUT"] == 0) {
                    $NRESCOPOMANUT = $this->newCode->getCodeByTableName("FCM_ESCOPOMANUT", $nrorg, 1, 10, true);

                    $parametrosEscopo = array(
                        "NRESCOPOMANUT"     => $NRESCOPOMANUT,
                        "NMESCOPOMANUT"     => $row['NMESCOPOMANUT'],
                        "NRORG"             => $nrorg,
                        "CDOPERINCLUSAO"    => $operador,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->ParametrizacaoFactory->insertEscopo($parametrosEscopo);
                } else {
                    $parametrosEscopo = array(
                        "NRESCOPOMANUT"     => $row['NRESCOPOMANUT'],
                        "NMESCOPOMANUT"     => $row['NMESCOPOMANUT'],
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->ParametrizacaoFactory->editEscopo($parametrosEscopo);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function getEscoposManutencao()
    {
        return $this->ParametrizacaoFactory->getEscoposManutencao();
    }

    public function deleteEscopos($params)
    {
        $parametro = implode(',', $params["NRS"]);
        $this->ParametrizacaoFactory->deleteEscopos($parametro);
    }

    /*
     * TAB UNIDADE DE MEDIDA
     */

    public function saveUnidadeMedida($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["ROWS"] as $row) {
                if ($row["NRUNIDADEMEDIDA"] == 0) {
                    $NRUNIDADEMEDIDA = $this->newCode->getCodeByTableName("FCM_UNIDADEMEDIDA", $nrorg, 1, 10, true);

                    $parametrosUnidade = array(
                        "NRUNIDADEMEDIDA"       => $NRUNIDADEMEDIDA,
                        "NMUNIDADEMEDIDA"       => $row['NMUNIDADEMEDIDA'],
                        "NMABREVUNIDADEMEDIDA"  => isset($row['NMABREVUNIDADEMEDIDA']) ? $row['NMABREVUNIDADEMEDIDA'] : null,
                        "NRORG"                 => $nrorg,
                        "CDOPERINCLUSAO"        => $operador,
                        "NRORGINCLUSAO"         => $nrorg,
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg
                    );

                    $this->ParametrizacaoFactory->insertUnidadeMedida($parametrosUnidade);
                } else {
                    $parametrosUnidade = array(
                        "NRUNIDADEMEDIDA"       => $row['NRUNIDADEMEDIDA'],
                        "NMUNIDADEMEDIDA"       => $row['NMUNIDADEMEDIDA'],
                        "NMABREVUNIDADEMEDIDA"  => $row['NMABREVUNIDADEMEDIDA'],
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg
                    );

                    $this->ParametrizacaoFactory->editUnidadeMedida($parametrosUnidade);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function getUnidadesMedida()
    {
        return $this->ParametrizacaoFactory->getUnidadesMedida();
    }

    public function deleteUnidadesMedida($params)
    {
        $parametro = implode(',', $params["NRS"]);
        $this->ParametrizacaoFactory->deleteUnidadesMedida($parametro);
    }

    public function getAbrevUniMedida()
    {
        return $this->ParametrizacaoFactory->getAbrevUniMedida();
    }
}
