import { Utilities } from '@/util/Utilities';
import {
    DialogService, Form, IterableComponentRender,
    LoadingService, Modal, ModalService, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata, Loader,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';

export class Fm00040004ModeloAuditoriaController {
    public AuditoriaController = Loader.getInstance('Fm00040000AuditoriaController');

    public modalCriacaoModelo!: Modal;

    private NRMODELOAUDIT: any = 0;

    public async criarModais() {
        this.modalCriacaoModelo = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00040_000_auditoria/modal/fm_00040_004_construir_modelo' });
    }

    public destruirModais() {
        this.modalCriacaoModelo?.destroy();
    }

    public abrirModal({ component }: any) {
        this.modalCriacaoModelo.show();
        const nr = component.name.split('_')[1];
        this.NRMODELOAUDIT = nr;

        setTimeout(() => {
            Metadata.getInstance<Form>('form-construir-modelo').clearForm();
        }, 200);
    }

    public fecharModal() {
        this.modalCriacaoModelo.hide();
    }

    public setFilterEstruturaModelo({ component }: IEventParam<TekGrid>) {
        component.datasource.setFilter({
            NRMODELOAUDIT: component.name.split('_')[1],
        });
    }

    public salvarModeloAuditoria() {
        const iterable = Metadata.getInstance<TekIterableComponentRender>('iterable-modelo-auditoria');

        LoadingService.show();
        Http.post('/salvarModeloAuditoria', {
            NMMODELOAUDIT: Metadata.getInstance<TextInput>('NMMODELOAUDIT').value,
        }).then(() => {
            iterable.datasource.get();
            Utilities.showSuccessMessage();
            this.AuditoriaController.fecharModalInserirModelo();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public changeTipoModelo({ component }: IEventParam<Select>) {
        const categoria = Metadata.getInstance<Select>('selecionar-categoria');
        const topicos = Metadata.getInstance<SelectMultiple>('selecionar-topicos');
        const criterios = Metadata.getInstance<SelectMultiple>('selecionar-criterios');

        if (component.value === 'C') {
            categoria.disabled = false;
            topicos.disabled = false;
            criterios.disabled = false;
            categoria.addValidation('required', {});
            topicos.removeValidation('required');
            criterios.addValidation('required', {});
        } else if (component.value === 'T') {
            categoria.disabled = false;
            topicos.disabled = false;
            criterios.disabled = true;
            categoria.addValidation('required', {});
            topicos.addValidation('required', {});
            criterios.removeValidation('required');
        }
    }

    public construirModelo() {
        const treeGrid = Metadata.getInstance<TekIterableComponentRender>('iterable-modelo-auditoria');

        Http.post('/salvarModelo', {
            NRCATEGORIAAUDIT: Metadata.getInstance<Select>('selecionar-categoria').value,
            NRMODELOAUDIT: this.NRMODELOAUDIT,
            NRTOPICOAUDIT: Metadata.getInstance<SelectMultiple>('selecionar-topicos').value,
            NRCRITERIOAUDIT: Metadata.getInstance<SelectMultiple>('selecionar-criterios').value,
        }).then(() => {
            treeGrid.datasource.get();
            Utilities.showSuccessMessage();
            this.fecharModal();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    private codigo: any = 0;

    public excluirModelo({ event, component }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040004ModeloAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
        const cod = component.name.split('_')[1];
        this.codigo = cod;
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<IterableComponentRender>('iterable-modelo-auditoria');

        LoadingService.show();
        Http.post('/excluirRegistros', {
            MODELO: this.codigo,
        }).then((response: IDictionary) => {
            if (response.data !== 0) {
                Utilities.showDialog('Operação Bloqueada', 'warning', 'Este Modelo de Auditoria está atualmente em curso e não pode ser excluído.');
            } else {
                Utilities.showSuccessMessage();
            }
            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public excluirDados({ event, component }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040004ModeloAuditoriaController.aposConfirmarDeleteDados', 'Excluir Registro(s)?');
        const cod = component.name.split('_')[1];
        this.codigo = cod;
    }

    public aposConfirmarDeleteDados() {
        const component = Metadata.getInstance<TekGrid>(`tek-tree-grid_${this.codigo}`);

        LoadingService.show();
        Http.post('/excluirRegistros', {
            DADOS: component.selectedRows.map((element: any) => element.CODIGO),
            MODELO: this.codigo,
        }).then((response: IDictionary) => {
            if (response.data !== 0) {
                Utilities.showDialog('Operação Bloqueada', 'warning', 'Os dados não podem ser excluídos, uma vez que estão sendo empregados em um Modelo de Auditoria atualmente em curso.');
            } else {
                Utilities.showSuccessMessage();
            }
            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
