import {
    Button, DialogService, FileInput, Form, IGridEvent, LoadingService, Modal, ModalService, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, Metadata,
} from '@zeedhi/core';
import { IterableCarousel, TekGrid } from '@zeedhi/teknisa-components-common';
import { ZdText } from '@zeedhi/vuetify';
import { Utilities } from '@/util/Utilities';
import FileUpload from '../FileUpload';

export class FmtAtivosInsumosAtivosController {
    private modalSalvarAtivo!: Modal;

    private modalAdicionarImagens!: Modal;

    private modalExcluirImagens!: Modal;

    private tituloModal: string = '';

    private carouselVisibility: boolean = false;

    private nrRowAtual: any;

    public async abrirModal() {
        this.modalSalvarAtivo = await ModalService.createFromJsonPost('', { jsonPath: '/fmt_ativosInsumos/modal/fmt_ativos_insumos_salvar_ativo' });
        this.modalSalvarAtivo.show();
    }

    public async abrirModalAdicionar() {
        this.tituloModal = 'Adicionar Ativo';
        await this.abrirModal();
    }

    public async abrirModalEdicao({ row }: IGridEvent) {
        this.modalSalvarAtivo?.destroy();
        this.tituloModal = 'Editar Ativo';
        await this.abrirModal();

        const inputNrRow = Metadata.getInstance<TextInput>('NRROW');
        const inputNome = Metadata.getInstance<TextInput>('InputNomeAtivo');
        const inputCodigo = Metadata.getInstance<TextInput>('InputCodigoAtivo');
        const selectCategoria = Metadata.getInstance<Select>('SelectCategoriaAtivo');
        const selectStatus = Metadata.getInstance<Select>('SelectStatusAtivo');
        const selectTag = Metadata.getInstance<SelectMultiple>('SelectTagAtivo');

        inputNrRow.value = row.NREQUIPAMENTO;
        inputNome.value = row.NMEQUIPAMENTO;
        inputCodigo.value = row.CDEQUIPAMENTO;
        selectCategoria.setValue(row.NRCATEGORIAEQUIP);
        selectStatus.setValue(row.IDSITUACAO);
        selectTag.setValue(1);
    }

    public fecharModal() {
        this.modalSalvarAtivo.hide();
        this.modalSalvarAtivo?.destroy();
    }

    public async salvarAtivo() {
        const form = Metadata.getInstance<Form>('form-salvar-ativo');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-ativos');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/salvarAtivo', {
                NRROW: form.value.NRROW,
                NMEQUIPAMENTO: form.value.InputNomeAtivo,
                CDEQUIPAMENTO: form.value.InputCodigoAtivo,
                NRCATEGORIAEQUIP: form.value.SelectCategoriaAtivo,
                IDSITUACAO: form.value.SelectStatusAtivo,
                ESCOPOS: form.value.SelectTagAtivo,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                this.fecharModal();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmtAtivosInsumosAtivosController.deleteAtivos', 'Excluir Ativos', 'Deseja realmente excluir os registros selecionados?');
    }

    public deleteAtivos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-ativos');

        LoadingService.show();
        Http.post('/deleteAtivos', {
            ROWS: grid.selectedRows.map((el) => el.NREQUIPAMENTO),
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            DialogService.hide();
            LoadingService.hide();
        });
    }

    public async abrirModalAdicionarImagens({ row }: any) {
        this.modalAdicionarImagens?.destroy();
        this.modalAdicionarImagens = await ModalService.createFromJsonPost('', { jsonPath: '/fmt_ativosInsumos/modal/fmt_ativos_insumos_adicionar_img_ativo' });
        this.nrRowAtual = row.NREQUIPAMENTO;
        this.modalAdicionarImagens.show();
    }

    public fecharModalAdicionarImagens() {
        this.modalAdicionarImagens.hide();
        this.modalAdicionarImagens?.destroy();
    }

    public async adicionarImagens() {
        const imgs = await this.updloadPictures('FileInputFotosAtivo');

        LoadingService.show();
        Http.post('/insertFotosAtivo', {
            NREQUIPAMENTO: this.nrRowAtual,
            FOTOS: imgs,
        }).then(() => {
            Utilities.showSuccessMessage();
            this.fecharModalAdicionarImagens();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            DialogService.hide();
            LoadingService.hide();
        });
    }

    public async abrirModalExcluirImagens({ row }: any) {
        this.modalExcluirImagens?.destroy();
        this.modalExcluirImagens = await ModalService.createFromJsonPost('', { jsonPath: '/fmt_ativosInsumos/modal/fmt_ativos_insumos_excluir_img_ativo' });
        this.nrRowAtual = row.NREQUIPAMENTO;
        this.modalExcluirImagens.show();
        setTimeout(() => {
            const carouselImagens = Metadata.getInstance<IterableCarousel>('carousel-imagens-ativo');
            this.carregarFotosAtivo({ component: carouselImagens });
        }, 300);
    }

    public carregarFotosAtivo({ component }: any) {
        const botaoApagar = Metadata.getInstance<Button>('botao-excluir-imagem');
        LoadingService.show();
        this.getFotosAtivo(this.nrRowAtual).then((response: any) => {
            if (response.data.data.length > 0) {
                component.iterable.datasource.data = [];
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRIMGEQUIPAMENTO: dados.NRIMGEQUIPAMENTO,
                        DSCAMINHO: dados.DSCAMINHO,
                    });
                });
            } else {
                const txtSemRes = Metadata.getInstance<ZdText>('sem-resultado-imagens-ativo');
                txtSemRes.isVisible = true;
                component.isVisible = false;
                botaoApagar.isVisible = false;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public getFotosAtivo(NRROW: any) {
        return Http.post('/getFotosAtivo', {
            NREQUIPAMENTO: NRROW,
        });
    }

    public excluirImagens() {
        const carousel = Metadata.getInstance<IterableCarousel>('carousel-imagens-ativo');
        const imgs = carousel.iterable.datasource.data;
        const [imgDeletada] = imgs.filter((value: any, index: any) => index === carousel.currentSlide);

        LoadingService.show();
        this.deleteFotoAtivo(imgDeletada).then(() => {
            const data = imgs.filter((value: any, index: any) => index !== carousel.currentSlide);
            carousel.iterable.datasource.updateData(data);
            if (data.length === 0) {
                const txtSemRes = Metadata.getInstance<ZdText>('sem-resultado-imagens-ativo');
                const botaoApagar = Metadata.getInstance<Button>('botao-excluir-imagem');
                txtSemRes.isVisible = true;
                botaoApagar.isVisible = false;
                carousel.isVisible = false;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public deleteFotoAtivo(imgDeletada: any) {
        return Http.post('/deleteFotoAtivo', {
            NRIMGEQUIPAMENTO: imgDeletada.NRIMGEQUIPAMENTO,
        });
    }

    public fecharModalExcluirImagens() {
        this.modalExcluirImagens.hide();
        this.modalExcluirImagens?.destroy();
    }

    public mostrarFotoAtivo({ component }: any) {
        const carouselFotos = Metadata.getInstance<IterableCarousel>('CarouselFotosAtivo');
        carouselFotos.iterable.datasource.updateData([]);
        const imagens: any = [];
        component.value.forEach((element: any, pos: any) => {
            imagens.push({
                id: `numero_${pos}`,
                src: window.URL.createObjectURL(element),
            });
        });
        carouselFotos.iterable.datasource.updateData(imagens);
        this.carouselVisibility = component.value.length > 0;
    }

    public async updloadPictures(name: string) {
        const fileInputObject = Metadata.getInstance<FileInput>(name).value;
        let arquivos: any = [];
        if (fileInputObject && fileInputObject.length > 0) {
            const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
            const buildFilesPromises = fileInputObject.map((file: any) => {
                if (types.includes(file.type)) {
                    return FileUpload.buildFileSendObject(file);
                }
                return [];
            });
            await Promise.all(buildFilesPromises).then((files: any) => {
                arquivos = files;
            });
          }
        return arquivos;
    }
}
